'use strict';
/*
 * Add date: {creation, modification, modified} and author: {name, email}
 * to metadata if not already present
 */

var nodegit = require('nodegit');
var path = require('path');
var moment = require('moment');

// Returns promise for patch of single commit
var gitShow = function(repo, commitData) {
  var commit, tree, parentTree;
  return nodegit.Revparse.single(repo, commitData.commit)
    .then((obj) => {
      return nodegit.Commit.lookup(repo, obj.id());
    })
    .then((_commit) => {
      commit = _commit;
      if(commit.parentcount() < 1) return null;
      return commit.getParents(1);
    })
    .then((parents) => {
      if(parents === null) return null;
      return parents[0].getTree();
    })
    .then((_parentTree) => {
      parentTree = _parentTree;
      return commit.getTree();
    })
    .then((_tree) => {
      tree = _tree;
      return nodegit.Diff.treeToTree(repo, parentTree, tree, null);
    })
    .then((diff) => {
      return diff.patches();
    })
    .then((patches) => {
      return [
        patches,
        commitData
      ];
    });
}

var gitlog = function(repo) {
  var map = function(repo){
    return Promise.resolve()
    .then(function(){
      return function(commit){
        var author = commit.author();
        var sha = commit.sha();

        return {
          'commit': sha,
          'author': {
            'name': author.name(),
            'email': author.email()
          },
          'date': commit.date(),
          'message': commit.message()
        };
      };
    });
  };


  return repo.getHeadCommit()
  .then(function(commit) {
    return new Promise(function(resolve, reject){
      commit.history()
      .on('end', resolve)
      .on('error', reject)
      .start();
    })
    .then(function(commits){
      return map(repo)
      .then(function(log){
        return commits.map(log);
      });
    });
  });
};

var getFilesHistory = function(repo, startCommitMessage) {
  if(!Array.isArray(startCommitMessage) && startCommitMessage !== undefined) {
    startCommitMessage = [startCommitMessage];
  }
  return Promise.resolve()
  .then(function() {
    return gitlog(repo)
  })
  .then(function(log){
    var promises = [];
    for(var i = 0; i < log.length; i++) {
      promises.push(gitShow(repo, log[i]));
    }
    return Promise.all(promises);
  })
  .then(function(diffs) {
    var filehistory = {};
    diffs.forEach(function(data) {
      var commit = data[1];
      var diff = data[0];

      diff.forEach(function(entry) {
        var p = path.normalize(entry.newFile().path());
        if(filehistory[p]===undefined) {
          filehistory[p] = [];
        }
        filehistory[p].push(commit);
      });
    });
    return filehistory;
  })
  .then(function(log) {
    if(startCommitMessage === undefined) return log;
    function mergeCommit(commits) {
      var r;
      commits.reverse();
      commits.forEach(function(commit) {
        if(r === undefined)
        startCommitMessage.forEach(function(matcher) {
          if(commit.message.indexOf(matcher) > -1) r = commit;
        })
      })
      return r;
    }
    for(var file in log) {
      log[file].mergeCommit = mergeCommit(log[file]);
    }
    return log;
  });
}

module.exports = function(obj) {
  console.log('Build step: GitInfo');
  let pathToRepo = path.resolve(obj.config.sourceDir);
  return new Promise(function(resolve,reject) {
    return nodegit.Repository.open(pathToRepo)
    .catch(function() {
      resolve(obj);
    })
    .then(function(repo) {
      return getFilesHistory(repo, obj.config.git.mergeMessage);
    })
    .then((filesHistory) => {
      obj.pages.forEach(function(article) {
        var file = path.relative(pathToRepo, article.filename);
        var commits = filesHistory[file];
        if(commits !== undefined) {
          commits.sort(function(a,b){
            return b.date - a.date;
          });
          var newestCommit = commits[0];
          var oldestCommit = commits[commits.length-1];
          var mergeCommit = commits.mergeCommit;

          if(article.metadata.date === undefined)
            article.metadata.date = {};

          if(article.metadata.date.creation === undefined) {
            if(mergeCommit === undefined)
              article.metadata.date.creation = moment();
            else
              article.metadata.date.creation = mergeCommit.date;
          }

          article.metadata.date.modification = newestCommit.date;
          article.metadata.date.modified = (newestCommit !== oldestCommit);

          if(article.metadata.author === undefined)
            article.metadata.author = {}
          if(article.metadata.author.name === undefined)
            article.metadata.author.name = oldestCommit.author.name
          if(article.metadata.author.email === undefined)
            article.metadata.author.email = oldestCommit.author.email

          article.commits = commits;
        } else {
          console.log('[Warning] Article '+ file +' is not in git repository or doesn\'t have merge commit');
        }
      });
      return obj;
    })
    .then(function(o) {
      resolve(o);
    })
    .catch(function(e){
      reject(e);
    });
  })
}
